from minidoc import minidoc
from minidoc import tst
from minidoc import comast
import argparse
from efdir import fs
import ast
from minidoc.comast import wfsdig
from minidoc.tst import creat_rst
import elist.elist as elel

parser = argparse.ArgumentParser()
parser.add_argument('-src','--src_file', default="",help=".py file name")
parser.add_argument('-codec','--codec', default="utf-8",help=".tst.py file codec")
parser.add_argument('-dst','--dst_file', default="",help="destination rst")
parser.add_argument('-proj','--proj_name', default="projname",help="project name")
parser.add_argument('-desc','--proj_desc', default="desc0\ndesc1",help="project description")
parser.add_argument('-lic','--proj_lic', default="MIT",help="project license")
parser.add_argument('-inst','--proj_inst_cmd', default="",help="project install cmd")
parser.add_argument('-tbot','--title_bot', default="=",help="parent title bottom char")
parser.add_argument('-ebot','--entry_bot', default="-",help="entry title bottom char")


def boolize(s):
    s = s.lower()
    if(s=="true"):
        return(True)
    elif(s=="false"):
        return(False)
    else:
        return(False)

def creat_usage_from_code(cd):
    root = ast.parse(cd)
    d = wfsdig(root)
    body = d['body']
    funcs = elel.cond_select_all(d['body'],cond_func=lambda ele:ele['_type'] == 'FunctionDef')
    kl = elel.mapv(funcs,lambda func:func['name'])
    vl = elel.mapv(funcs,lambda func:[func['body'][0]['value']['s']])
    usage = creat_rst(kl,vl)
    return(usage)

args = parser.parse_args()


def main():
    projhd = tst.creat_projhd(args.proj_name,args.proj_desc)
    install = tst.creat_install(args.proj_name)
    license = tst.creat_license(args.proj_lic)
    src = (args.proj_name+".py") if(args.src_file=="") else args.src_file
    cd = fs.rfile(src)
    usage = creat_usage_from_code(cd)
    rst_str = projhd + install + license + usage
    dst = (args.proj_name+".rst") if(args.dst_file=="") else args.dst_file
    fs.wfile(dst,rst_str,codec=args.codec)



#minidoc_from_comments



import docutils.core
import docutils.parsers.rst


'''
Engine
   
| #rst_str            rst-string
| #html_str           html-string
| #xml_str            xml-string
| #tree               docutils.nodes.document-tree
| #r                  rst_str
| #t..................docutils.nodes.document-tree
| #h                  html_str
| #x                  xml_str
| #r2h                rst_str-to-html_str
| #r2t                rst_str-to-docutils.nodes.document_tree
| #t2x                docutils.nodes.document_tree-to-rst_str
| #r2x            rst_str-to-xml_str
'''


def r2h(rst_str):
    '''
    | convet rst_str to html_str,which includes CSS in it.
    
    :param rst_str: string.
        rst-string.
    :return: string.
        html-string.
    
    '''
    html_dict = docutils.core.publish_parts(writer_name='html',source=rst_str)
    return(html_dict['whole'])

def r2t(rst_str,name=''):
    '''
    | convert rst_str to docutils.nodes.document_tree
    
    :param rst_str: string.
        rst_str.
    :param name: string.
        the source attribute of document node of the tree,this is not important,any html escaped string is OK.
        Default is ""
    :return: docutils.nodes.document.
        doc-tree-root-node
    '''
    parser = docutils.parsers.rst.Parser()
    doc = docutils.utils.new_document(name)
    doc.settings.tab_width = 4
    doc.settings.pep_references = 1
    doc.settings.rfc_references = 1
    doc.settings.syntax_highlight = 1
    doc.settings.trim_footnote_reference_space = 1
    parser.parse(rst_str, doc)
    return(doc)

def t2x(tree):
    '''
    | convert docutils.nodes.document_tree  to xml 
    
    :param docutils.nodes.document.
        doc-tree-root-node
    
    :return: string.
        xml_str
    '''
    xdom = tree.asdom()
    xml_str = xdom.toprettyxml()
    xml_str = xml_str.replace("\t","\x20"*4)
    return(xml_str)

def r2x(rst_str,name=''):
    '''
    | convert rst_str to xml_str
    
    :param rst_str: string.
        rst_str.
    :param name: string.
        the source attribute of document node of the tree,this is not important,any html escaped string is OK.
        Default is ""
    :return: string.
        xml_str
    '''
    tree = r2t(rst_str,name)
    xml_str = t2x(tree)
    return(xml_str)


#####



import ast
import astunparse
from efdir import fs
import ast
import astunparse
import elist.elist as elel

def get_node_type(nd):
    return(type(nd).__name__)

def is_leaf(v):
    cond0 = (isinstance(v,list) and len(v)==0)
    cond1 = not(is_node(v))
    cond = true if(cond0) else cond1
    return(cond)

def is_node(v):
    s  = v.__str__()
    if('_ast.' in s):
        return(True)
    else:
        return(False)

def nd2d(nd):
    d = {}
    g = ast.iter_fields(nd)
    d['_type'] = get_node_type(nd)
    for it in g:
        d[it[0]] = it[1]
    return(d)

def is_children(v):
    if(is_node(v)):
        return(True)
    elif(isinstance(v,list) and len(v)>0):
        return(True)
    else:
        return(False)

def is_list_children(v):
    return(isinstance(v,list) and len(v)>0)

def get_children(d):
    children = []
    for k in d:
        v = d[k]
        if(is_list_children(v)):
            v = elel.mapv_inplace(v,nd2d)
            d[k] = v
            children.extend(v)
        elif(is_node(v)):
            v = nd2d(v)
            d[k] = v
            children.append(v)
        else:
            pass
    return(children)



def wfsdig(root):
    d = nd2d(root)
    unhandled = [d]
    while(unhandled.__len__()>0):
        next_unhandled = []
        for i in range(unhandled.__len__()):
            ele = unhandled[i]
            children = get_children(ele)
            next_unhandled.extend(children)
        unhandled = next_unhandled
    return(d)




import os
import termtosvg.main as term
import tempfile
import elist.elist as elel
from efdir import fs
#efdir >0.0.14
import pty
import time
from multiprocessing import Process
import estring.estring as eses
#estring > 0.6

def get_last_svg(output_path):
    svgs = fs.walkf(output_path)
    svgs.sort()
    if(svgs.__len__()>1):
        return(svgs[-2])
    else:
        print("something wrong~!")
        return(svgs[-1])

def write2slave(s,slave_fd,**kwargs):
    if("codec" in kwargs):
        codec = kwargs['codec']
    else:
        codec = 'utf-8'
    if("input_speed" in kwargs):
        input_speed = kwargs['input_speed']
    else:
        input_speed = 7
    delay = 1 / input_speed
    for c in s:
        os.write(slave_fd,c.encode(codec))
        time.sleep(delay)
    os.close(slave_fd)


def creat_svg(cmds_str,**kwargs):
    if('bin_path' in kwargs):
        bin_path = kwargs['bin_path']
    else:
        bin_path = '/usr/local/bin/termtosvg'
    if('py' in kwargs):
        py = kwargs['py']
    else:
        py = "python3"
    if('codec' in kwargs):
        codec = kwargs['codec']
    else:
        codec = 'utf-8'
    if('still_frames' in kwargs):
        still_frames = kwargs['still_frames']
    else:
        still_frames = False
    if('screen_size' in kwargs):
        screen_size = kwargs['screen_size']
    else:
        screen_size = "160x50"
    if(still_frames):
        output_path = tempfile.mkdtemp(prefix='termtosvg_')
        input_fileno,slave_fd = eses.str2io(cmds_str,codec=codec)
    else:
        _, output_path = tempfile.mkstemp(prefix='termtosvg_',suffix='.svg')
        input_fileno,slave_fd = pty.openpty()
    if(still_frames):
        print("----")
        print(screen_size)
        print("----")
        term.main([bin_path, output_path,'-c', py, '-s','-g',screen_size],input_fileno,None)
        output_path = get_last_svg(output_path)
        os.close(slave_fd)
        os.close(input_fileno)
    else:
        p_slave = Process(target=write2slave, args=(cmds_str,slave_fd,))
        p_slave.start()
        term.main([bin_path, output_path,'-c', py,'-g',screen_size],input_fileno,None)
        os.close(input_fileno)
        p_slave.join()
    print("final file at "+output_path)
    return(output_path)


def get_screen_size(arr,**kwargs):
    if("rownums" in kwargs):
        rownums = kwargs['rownums']
    else:
        rownums = 30
    if("colnums" in kwargs):
        colnums = kwargs['colnums']
    else:
        colnums = elel.max_length(arr) + 5 
    return(str(colnums)+"x"+str(rownums))

def cmds_arr2str(arr,**kwargs):
    '''
    '''
    if('still_frames' in kwargs):
        still_frames = kwargs['still_frames']
    else:
        still_frames = True
    s = elel.join(arr,"\r")
    s = "\r" + s + "\r"  + "exit()" + "\r"
    return(s)






from tokenize import tokenize, untokenize, COMMENT,NL,NEWLINE
from efdir import fs
from io import BytesIO
import estring.estring as eses
import re
import elist.elist as elel

# tokenize.NL
# Token value used to indicate a non-terminating newline. 
# The NEWLINE token indicates the end of a logical line of Python code; 
# NL tokens are generated when a logical line of code is continued over multiple physical lines.

def _group2str(group):
    s = untokenize(group)
    s = eses.strip(s,"\\\n",mode="whole")
    return(s)

def _infos2group(infos):
    def map_func(ele):
        ele.pop(0)
        return(ele)
    indexes = elel.cond_select_indexes_all(infos,cond_func=lambda info:(info.type==NEWLINE))
    groups = elel.broken_seqs(infos,indexes)
    groups = elel.remove_all(groups,[])
    groups = elel.mapv(groups,map_func)
    groups = elel.remove_all(groups,[])
    groups = elel.mapv(groups,_group2str)
    try:
        groups = elel.remove_last(groups,"\n")
        groups = elel.remove_first(groups,"\n")
    except:
        pass
    else:
        pass
    return(groups)


def tst2kvlist(**kwargs):
    if("codec" in kwargs):
        codec = kwargs["codec"]
    else:
        codec = "utf-8"
    if("code" in kwargs):
        code = kwargs["code"]
    else:
        code = fs.rfile(kwargs["fn"])
    code = code + "\n#"
    g = tokenize(BytesIO(code.encode(codec)).readline)
    info = next(g)
    kl = []
    vl = []
    k = ""
    infos = [info]
    for info in g:
        if(info.type == COMMENT):
            kl.append(k)
            v = _infos2group(infos)
            vl.append(v)
            k = info.string.lstrip("#")
            infos = []
        else:
            infos.append(info)
    indexes = elel.cond_select_indexes_all(kl,cond_func=lambda ele:ele=="")
    kl = elel.pop_indexes(kl,indexes)['list']
    vl = elel.pop_indexes(vl,indexes)['list']
    return((kl,vl))


#####


def arr2str(cmds_arr):
    return(elel.join(cmds_arr,"\n"))

def creat_head(s,i=None,bot="-"):
    if(i==None):
        pass
    else:
        s = str(i) + ". " + s
    return(s+"\n"+bot*s.__len__()+"\n\n")

def creat_title(title="Usage",bot="="):
    return(creat_head(title,i=None,bot=bot))

def creat_code_blk(cmds_arr):
    s = arr2str(cmds_arr)
    arr = s.split("\n")
    head1 = "    ::"
    head2 = "    "
    arr = elel. mapv(arr,lambda s:("        "+s))
    arr = elel.prepend(arr,head2)
    arr = elel.prepend(arr,head1)
    s = elel.join(arr,"\n") + "\n\n"
    return(s)


def get_svg_name(k):
    regex = re.compile("(.*?)\(.*\)")
    m = regex.search(k)
    if(m):
        return(m.group(1))
    else:
        return(k)

def creat_one_rst(k,v,i=None,**kwargs):
    if('bot' in kwargs):
        bot = kwargs['bot']
    else:
        bot = "-"
    head = creat_head(k,i,bot=bot)
    body = creat_code_blk(v)
    name = get_svg_name(k)
    image = ".. image:: ./images/" + name+".svg"+"\n\n"
    return(head+body+image)

def creat_rst(kl,vl,**kwargs):
    if("title" in kwargs):
        title = kwargs['title']
    else:
        title = "Usage"
    if("title_bot" in kwargs):
        title_bot = kwargs['title_bot']
    else:
        title_bot = "="
    if("entry_bot" in kwargs):
        entry_bot = kwargs['entry_bot']
    else:
        entry_bot = "-"
    title = creat_title(title=title,bot=title_bot)
    sarr = []
    for i in range(kl.__len__()):
        k = kl[i]
        v = vl[i]
        s = creat_one_rst(k,v,i=i,bot=entry_bot)
        sarr.append(s)
    s = title + elel.join(sarr,"")
    return(s)


#####

#####
#creat  from  comments
#####

def creat_projhd(projname,desc):
    l0 = ".. contents:: Table of Contents\n"
    l1 = "   :depth: 5\n\n\n"
    l2 = "*"+projname+"*\n"
    l3 = "=" * (len(l2) - 1) + "\n"
    descs = desc.split("\n")
    descs = elel.mapv(descs, lambda ele:"- "+ele)
    l4 = elel.join(descs,"\n")
    l5 = "\n\n\n"
    s = l0+l1+l2+l3+l4+l5
    print(s)
    return(s)


def creat_install(*args):
    l0 = "Installation\n"
    l1 = "------------\n"
    l2 = "\n    ::\n"
    l3 = "\n"
    projname = args[0]
    l4 = "        $ "
    l5 = "pip3 install " if(len(args)==1) else args[1]
    l6 = projname
    l7 = "\n\n"
    s = l0+l1+l2+l3+l4+l5+l6+l7
    print(s)
    return(s)



def creat_license(*args):
    lic = "MIT" if(len(args)==0) else args[0]
    tem = "License\n-------\n\n- " + lic + "\n\n"
    print(tem)
    return(tem)





from minidoc import svg
from minidoc import tst
from efdir import fs
import shutil
import os

def creat_one_svg(k,v,i=None,**kwargs):
    if("dst_dir" in kwargs):
        dst_dir = kwargs['dst_dir']
    else:
        dst_dir = "./images"
    screen_size = svg.get_screen_size(v,**kwargs)
    kwargs['screen_size'] = screen_size
    cmds_str = svg.cmds_arr2str(v,**kwargs)
    output_path = svg.creat_svg(cmds_str,**kwargs)
    #name = tst.get_svg_name(k) + "." + str(i) + ".svg"
    name = tst.get_svg_name(k) + ".svg"
    dst = os.path.join(dst_dir,name)
    shutil.move(output_path,dst)
    return(dst)


#still_frames
#rownums
#colnums

def creat_svgs(kl,vl,**kwargs):
    if("dst_dir" in kwargs):
        dst_dir = kwargs['dst_dir']
    else:
        dst_dir = "./images"
    fs.mkdir(dst_dir)
    arr = []
    for i in range(kl.__len__()):
        k = kl[i]
        v = vl[i]
        dst = creat_one_svg(k,v,i=i,**kwargs)
        arr.append(dst)
    return(arr)


####
####








from minidoc import minidoc
from minidoc import tst
import argparse
from efdir import fs

parser = argparse.ArgumentParser()
parser.add_argument('-tst','--test_file', default="code.tst.py",help=".tst.py file name")
parser.add_argument('-codec','--codec', default="utf-8",help=".tst.py file codec")
parser.add_argument('-still','--still_frames', default="True",help="generate screen shot")
parser.add_argument('-rows','--rownums', default="30",help="screen height")
parser.add_argument('-dst','--dst_dir', default="./images",help="destination svg dir")
parser.add_argument('-title','--title', default="Usage",help="parent title")
parser.add_argument('-tbot','--title_bot', default="=",help="parent title bottom char")
parser.add_argument('-ebot','--entry_bot', default="-",help="entry title bottom char")


def boolize(s):
    s = s.lower()
    if(s=="true"):
        return(True)
    elif(s=="false"):
        return(False)
    else:
        return(False)

args = parser.parse_args()
still_frames = boolize(args.still_frames)


def main():
    kl,vl = tst.tst2kvlist(fn=args.test_file,codec=args.codec)
    minidoc.creat_svgs(kl,vl,still_frames=still_frames,rownums=int(args.rownums),dst_dir=args.dst_dir)
    rst_str = tst.creat_rst(kl,vl,title=args.title,title_bot=args.title_bot,entry_bot=args.entry_bot)
    fs.wfile(args.title+".rst",rst_str,codec=args.codec)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""The setup script."""

from setuptools import setup, find_packages

try:
    with open('./README.rst') as readme_file:
        readme = readme_file.read()
except:
    readme = "refer to https://github.com/ihgazni2/minidoc/"
else:
    pass

requirements = [
    "termtosvg",
    "efdir",
    "estring",
    "elist"
]

setup_requirements = [
    "termtosvg",
    "elist",
    "efdir",
    "estring"
]


setup(
    name='minidoc',
    version='0.0.4',
    description="auto generate a mini rst doc (code block + svg) from a test-file",
    long_description=readme,
    author="dli",
    author_email='286264978@qq.com',
    url='https://github.com/ihgazni2/minidoc',
    packages=find_packages(),
    package_data={
                  'documentation': ['docs/*']
                 },
    include_package_data=True,
    install_requires=requirements,
    license="MIT license",
    keywords='doc,test,svg',
    entry_points = {
         'console_scripts': [
             'minidoc=minidoc.bin:main',
             'minidoc_from_docstring=minidoc.BINS.minidoc_from_docstring:main'
         ]
    },
    classifiers=[
        'Environment :: Console',
        'Environment :: Web Environment',
        'Intended Audience :: End Users/Desktop',
        'Intended Audience :: Developers',
        'Intended Audience :: System Administrators',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
    ],
    setup_requires=setup_requirements,
    py_modules=['minidoc'],
)
